#include <libporty/net/network.h>

/* Run background process binding stdio to fd_*, stderr is left alone and
   all other fds are closed. fd_* will be created by the call (they
   are output parameters) and neither of them may be NULL.

   argv[0] is the command to be run followed by all the arguments
   (argv[1] doesn't repeat the process name but is the first actual argument
   of the process called); terminated by a NULL entry.

   ctrlfd is an optional control fd; using this the caller may monitor
   exit status of the process or kill it.

   FDs as the child process sees:
   fd_stdin  becomes child fd 0
   fd_stdout becomes child fd 1
   child fd 2 is the same as parent fd 2
*/
int P_net_async_bgproc(char *argv[], P_net_socket *ctrlfd, P_net_socket *fd_stdin, P_net_socket *fd_stdout);

/* Same as P_net_async_bgproc but instead of hardwiring stdio/stderr bindings,
   inherit listed P_net_sockets (inherit_len is the number of P_net_sockets to
   be inherited). 

   Note: unlike in P_net_async_bgproc, fds pipes shall be created by the caller.
   Passing a non-pipe socket is also possible, but each parent socket must be
   passed only once and each children fd shall be referenced only once. This
   is a limitation imposed by some of the target systems.

   Note 2: when passing a parent fd to a child process, it may be closed on
   the parent side - the parent shall not rely on accessing the same fd
   concurrent with the child process.
*/
int P_net_async_bgproc_inherit(char *argv[], P_net_socket *ctrlfd, P_net_socket *parent_fds, int *child_fds, int inherit_len);


/* --- for internal use, for the wrapper --- */

#ifndef WIN32
/* close all irrelevant fds leaving stdio only */
void P_net_async_bgproc_closefds(P_net_socket *inherit_list, int inherit_len);
#endif
